// ref: https://github.com/GPUOpen-Tools/device_info/blob/0f9cb0d126a1345331e71116733553602b7585d7/DeviceInfo.cpp
pub const AMDGPU_IDS_2: &[(u32, u32, &str)] = &[
    (0x1586, 0xC3, "AMD Radeon 8060S Graphics"),
    (0x1586, 0xC6, "AMD Radeon 8060S Graphics"),
    (0x1902, 0xC0, "AMD Radeon 840M Graphics"),
    (0x1902, 0xC6, "AMD Radeon 820M Graphics"),
    (0x1902, 0xC7, "AMD Radeon 840M Graphics"),
    (0x1902, 0xC8, "AMD Radeon 840M Graphics"),
    (0x1902, 0xD7, "AMD Radeon 840M Graphics"),
    (0x1902, 0xD8, "AMD Radeon 840M Graphics"),
];
